'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _commonStreams = require('common-streams');

var _mississippi = require('mississippi');

var _mississippi2 = _interopRequireDefault(_mississippi);

var _maybedone = require('maybedone');

var _maybedone2 = _interopRequireDefault(_maybedone);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const concat = _mississippi2.default.concat;
const pipe = _mississippi2.default.pipe;


const globalDebug = (0, _debug2.default)('store-uploader');

const randomKey = () => _uuid2.default.v1();

// Store is an abstract-blob-store
// Uploads a file to AWS S3 (or other blob store)
// and returns { url, hashes, size }

exports.default = function (store) {
  var _ref = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var _ref$keyPrefix = _ref.keyPrefix;
  let keyPrefix = _ref$keyPrefix === undefined ? 'uploads/' : _ref$keyPrefix;
  var _ref$algos = _ref.algos;
  let algos = _ref$algos === undefined ? ['sha1', 'sha256', 'md5'] : _ref$algos;
  return function (readStream) {
    var _ref2 = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    let contentType = _ref2.contentType;
    return Promise.resolve().then(() => {
      const key = [keyPrefix, randomKey()].join('');
      const debug = function debug() {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return globalDebug(key, ...args);
      };

      const hashStreams = algos.map(algo => [algo, new _commonStreams.HashStream(algo)]);
      const sizeStream = new _commonStreams.SizeStream();

      debug('Starting upload... ');

      return new Promise((resolve, reject) => {
        const result = { key, digests: {} };

        const cb = (0, _maybedone2.default)(err => {
          if (err) return reject(err);
          return resolve(result);
        }, 3 + hashStreams.length);

        const storeStream = store.createWriteStream({
          key,
          contentType
        }, cb);

        const saveDigest = algo => concat(_ref3 => {
          var _ref4 = _slicedToArray(_ref3, 1);

          let digest = _ref4[0].digest;

          result.digests[algo] = digest;
        });

        const saveSize = () => concat(_ref5 => {
          var _ref6 = _slicedToArray(_ref5, 1);

          let size = _ref6[0].size;

          result.size = size;
        });

        hashStreams.forEach(_ref7 => {
          var _ref8 = _slicedToArray(_ref7, 2);

          let algo = _ref8[0];
          let hashStream = _ref8[1];

          pipe(readStream, hashStream, saveDigest(algo), err => {
            if (err) return cb(err);
            cb();
          });
        });
        pipe(readStream, sizeStream, saveSize(), err => {
          if (err) return cb(err);
          cb();
        });
        pipe(readStream, storeStream, err => {
          if (err) return cb(err);
          cb();
        });
      }).then(result => {
        debug('Upload done');
        return result;
      });
    });
  };
};